# coding: Shift_JIS

require 'pathname'
require 'fileutils'
require 'vr/vruby'
require 'vr/vrdialog'
require 'vr/vrcontrol'

module TeXworksMod
	class SetForm < VRModalDialog
		def construct
			left,right,top,bottom = AbTeXinst::get_windowpos(hParent);
			move(left,top,420,300);
			@font = @screen.factory.newfont("lr oSVbN",12);
			self.caption = 'TeXworks ݒ';
			addControl(VRCheckbox,'shortcutchk','TeXworks ̃V[gJbgfXNgbvɍ쐬',10,10,500,20);
			@shortcutchk.setFont(@font);
			addControl(VRStatic,'static','TeXworks pɖߍރtHgw肵ĂD',10,40,500,20);
			@static.setFont(@font);
			addControl(VRRadiobutton,'default','ftHg̎wg܂D',20,60,500,20);
			@default.setFont(@font);
			addControl(VRRadiobutton,'msmingoth','msmingoth.map ilr Elr SVbNp܂Dj',20,80,500,20);
			@msmingoth.setFont(@font);
			addControl(VRRadiobutton,'hiragino','hiragino.map iqMmtHgp܂Dj',20,100,500,20);
			@hiragino.setFont(@font);
			addControl(VRRadiobutton,'ipaex','ipaex.map iIPAex tHgp܂Dj',20,120,500,20);
			@ipaex.setFont(@font);
			addControl(VRRadiobutton,'kozuka','kozuka.map i˖ESVbN Pro p܂Dj',20,140,500,20);
			@kozuka.setFont(@font);
			addControl(VRRadiobutton,'kozukapr6n','kozukapr6n.map i˖ESVbN Pro6N p܂Dj',20,160,500,20);
			@kozukapr6n.setFont(@font);
			addControl(VRRadiobutton,'noembed','noembed.map iߍ݂܂DTeXworks ł͂lr ݂̂ŕ\Dj',20,180,500,20);
			@noembed.setFont(@font);
			addControl(VRCheckbox,'jischk','2004JIS `gDiftHg̎wI͖܂Dj',20,200,500,20);
			@jischk.setFont(@font);
			addControl(VRButton,'OK','OK',305,230,30,20);
			@OK.setFont(@font);
			addControl(VRButton,'Cancel','ݾ',345,230,60,20);
			@Cancel.setFont(@font);
			
			set_data(options['embed'],options['shortcut'],options['jis']);
	 	end
	 	
	 	def set_data(emb,shortcut,jis)
	 		@embed = emb;
	 		all_check_clear;
			case emb
			when 'msmingoth'
				@msmingoth.check(true);
			when 'hiragino'
				@hiragino.check(true);
			when 'ipaex'
				@ipaex.check(true);
			when 'kozuka'
				@kozuka.check(true);
			when 'kozukapr6n'
				@kozukapr6n.check(true);
			when 'noembed'
				@noembed.check(true);
			else
				@default.check(true);
			end
			@shortcut = shortcut;
			@shortcutchk.check(shortcut);
			@jis = jis;
			@jischk.check(jis);
	 	end
	 	
	 	def OK_clicked
	 		if @jischk.checked? && (get_embed_status == 'ipaex' || get_embed_status == 'msmingoth') then
	 			AbTeXinst::message_box(0,get_embed_status + '  2004JIS `͂܂D');
	 			return;
	 		end
	 		@embed = get_embed_status;
	 		if @embed == nil then @embed = 'default'; end
	 		@shortcut = @shortcutchk.checked?;
	 		@jis = @jischk.checked?;
	 		close([@embed,@shortcut,@jis]);
	 	end
	 	
	 	def Cancel_clicked
	 		close(nil);
	 	end

	private
		def get_embed_status
			if @default.checked? then return "default"; end
			if @msmingoth.checked? then return "msmingoth"; end
			if @hiragino.checked? then return "hiragino"; end
			if @ipaex.checked? then return "ipaex"; end
			if @kozuka.checked? then return "kozuka"; end
			if @kozukapr6n.checked? then return "kozukapr6n"; end
			if @noembed.checked? then return "noembed"; end
			return nil;
		end

		def all_check_clear
			@default.check(false);
			@msmingoth.check(false);
			@hiragino.check(false);
			@ipaex.check(false);
			@kozuka.check(false);
			@kozukapr6n.check(false);
			@noembed.check(false);
		end

	end



	class TeXworks
		def initialize
			@ini = "";
			@log = "";
			@inisec = 'plugin_TeXworks';
		end
		
		def info
			return [{"softname" => "TeXworks", "info" => "TeXworks ̐ݒ"}];
		end
		
		
		def plugin_init(ini,log)
			@ini = ini;
			@log = log;
			@embed = AbTeXinst::get_ini_str(ini,@inisec,'embed');
			if @embed == nil || @embed == "" then
				@embed = 'default';
			end
			sc = AbTeXinst::get_ini_str(ini,@inisec,'shortcut');
			@shortcut = false;
			if sc == nil || sc == "" || sc == 'true' then
				@shortcut = true;
			end
			j = AbTeXinst::get_ini_str(ini,@inisec,'jis');
			if j == 'true' then @jis = true; else @jis = false; end;
		end
		
		
		def reboot?()
			return false;
		end
		
		
		def set?(soft)
			return true;
		end

		
		def set(hwnd,soft)
			par = SWin::LWFactory.new(SWin::Application.hInstance).newwindow(nil,SWin::Window);
			AbTeXinst::sethwndtovrobj(par,hwnd);
			rv = VRLocalScreen.modalform(par,nil,SetForm,nil,{'embed'=>@embed,'shortcut'=>@shortcut,'jis' => @jis});
			if rv then @embed,@shortcut,@jis = rv; end
			return;
		end
		
		
		def get_info(hwnd)
			return true;
		end
		
		
		def download_data(soft)
			return {
				"success" => true,
				"download_files" => []};
		end
		
		
		def default?(soft)
			return (
				(@shortcut && !File.exist?(AbTeXinst::get_specialfolder(AbTeXinst::CSIDL_DESKTOP) + '\TeXworks.lnk')) || 
				(@embed != 'default' && !File.exist?(AbTeXinst::Setting::install_dir + "bin\\pdfplatex_#{@embed}.bat"))
			)
		end
		
		
		def install(hwnd,soft,dlresult)
			texworksexe = AbTeXinst::Setting::install_dir + 'share\texworks\TeXworks.exe';
			
			if !File.exist?(texworksexe) then
				Log("TeXworksCXg[ĂȂ悤łD");
				AbTeXinst::add_message(hwnd,"TeXworks̔Ɏs\r\n");
				return false;
			end

			AbTeXinst::add_message(hwnd,'TeXworks̐ݒcc\r\n');
			result = true;
			if @shortcut then
				Log("TeXworks̃V[gJbg쐬");
				if AbTeXinst::create_shortcut(texworksexe,'',AbTeXinst::Setting::install_dir + 'share\texworks',AbTeXinst::get_specialfolder(AbTeXinst::CSIDL_DESKTOP) + '\TeXworks.lnk') == true then
					msg = "";
				else
					msg = "s";
					result = false;
				end
				Log("TeXworks̃V[gJbg쐬" + msg);
			end
			
			AbTeXinst::add_message(hwnd,'TeXworksp̃ob`t@C쐬');
			Dir.glob(File.expand_path(AbTeXinst::Setting::install_dir + 'share/texmf-dist/fonts/map/dvipdfmx/base/*.map')).each{|mapfile|
				if /cid|base14|omegaj|ckx/ =~ mapfile then next;end
				mapfile = Pathname(mapfile).basename.to_s.gsub(/\.map$/,'');
				begin
					fp = File.open(AbTeXinst::Setting::install_dir + 'bin\pdfplatex_' + mapfile + '.bat','w');
					fp.write("@echo off\nplatex -synctex=1 -jobname=\"%~n1\" -kanji=utf8 -guess-input-enc %1 && ^\ndvipdfmx -f #{mapfile}.map \"%~n1\"\n");
#					fp.write("@echo off\nplatex -synctex=1 -jobname=\"%~n1\" %1 && ^\ndvipdfmx -f #{mapfile}.map \"%~n1\"\n");
					Log("pdfplatex_#{mapfile}.bat̐ɐ");
				rescue => excep
					Log("pdfplatex_#{mapfile}.bat̐Ɏs " + excep.to_s);
					result = false;
				ensure
					if fp then fp.close(); end
				end
			}
			pdfplatexbat = '';
			case @embed
			when 'default'
				pdfplatexbat = 'pdfplatex.bat';
			else
				if @jis == true then
					pdfplatexbat = 'pdfplatex_' + @embed + '04.bat';
				else
					pdfplatexbat = 'pdfplatex_' + @embed + '04.bat';
				end
			end
			
			if !File.exist?(AbTeXinst::Setting::install_dir + "bin\\#{pdfplatexbat}") then
				Log(pdfplatexbat + 'Ȃ悤łD');
				AbTeXinst::add_message(hwnd,"s\r\n");
				return false;
			end
			
			AbTeXinst::add_message(hwnd,'TeXworks̐ݒύX');
			ini = AbTeXinst::Setting::install_dir + 'share\texworks\twdata\configuration\tools.ini';

			set_done = false;
			1.upto(100){|n|
				sec = sprintf('%03d',n);
				if AbTeXinst::get_ini_str(ini,sec,'name') == 'pdfpLaTeX' then
					if AbTeXinst::write_ini_str(ini,sec,'program',pdfplatexbat) then
						set_done = true;
						Log("pdfpLaTeX̐ݒ#{pdfplatexbat}ɕύX");
						break;
					end
				end
			}
			
			if !set_done then
				Log('TeXworks/pdfpLaTeX̐ݒύXɎs');
				result = false;
			end
			
			
#			if set_done then
#				n = 1;
#				name = '';
#				noembed_found = false;
#				begin
#					name = AbTeXinst::get_ini_str(ini,sprintf('%03d',n),'name');
#					if name == 'pdfpLaTeX_noEmbed' then
#						noembed_found = true;
#						break;
#					end
#					n += 1;
#				end while name != ''
#				if !noembed_found then
#					sec = sprintf('%03d',n);
#					AbTeXinst::write_ini_str(ini,sec,'name','pdfpLaTeX_noEmbed');
#					AbTeXinst::write_ini_str(ini,sec,'program','pdfplatex.bat');
#					AbTeXinst::write_ini_str(ini,sec,'arguments','$fullname');
#					AbTeXinst::write_ini_str(ini,sec,'showPdf','true');
#				end
#			else
#				Log('TeXworks/pdfpLaTeX̐ݒύXɎs');
#				result = false;
#			end
			
			AbTeXinst::add_message(hwnd,(result ? '' : 's') + "\r\n");
			return result;
		end
		
		def plugin_uninit
			if AbTeXinst::Setting::write_ini? then
				AbTeXinst::write_ini_str(@ini,@inisec,'embed',@embed);
				AbTeXinst::write_ini_str(@ini,@inisec,'shortcut',@shortcut ? 'true' : 'false');
				AbTeXinst::write_ini_str(@ini,@inisec,'jis',@jis ? 'true' : 'false');
			end
		end
		
	private
		def Log(l)
			begin
				fp = File.open(@log,"a+");
			rescue
				return;
			end
			fp.write(l + "\n");
			fp.close();
		end
		
	end

end

AbTeXinst::register_plugin("TeXworks",TeXworksMod::TeXworks);

